import re
from subprocess import check_output, CalledProcessError

from jinja2 import Environment, PackageLoader

from s3pypi import __prog__


class Package(object):

    def __init__(self, name):
        self.name, self.version = name.split('-')
        self.filename = '%s.tar.gz' % name

    def __str__(self):
        return '%s-%s' % (self.name, self.version)

    def _attrs(self):
        return self.name, self.version, self.filename

    def __lt__(self, other):
        return self.version < other.version

    def __eq__(self, other):
        return isinstance(other, Package) and self._attrs() == other._attrs()

    def __hash__(self):
        return hash(self._attrs())

    @staticmethod
    def create():
        try:
            stdout = check_output(['python', 'setup.py', 'sdist', '--formats', 'gztar']).strip()
        except CalledProcessError as e:
            raise RuntimeError(e.output.rstrip())

        match = re.search('^making hard links in (.+)\.\.\.$', stdout, flags=re.MULTILINE)

        if match:
            return Package(match.group(1))
        else:
            raise RuntimeError(stdout)


class Index(object):

    template = Environment(loader=PackageLoader(__prog__, 'templates')).get_template('index.html.j2')

    def __init__(self, url, packages):
        self.packages = set(packages)
        self.url = url

    @staticmethod
    def parse(url, html):
        return Index(url, (Package(m[1]) for m in re.findall('<a href="(.+)">(.+)</a>', html)))

    def to_html(self):
        return self.template.render({'url': self.url, 'packages': sorted(self.packages, reverse=True)})



import os

import boto
from boto.exception import S3ResponseError
from boto.s3.key import Key

from s3pypi.package import Index


class S3Storage(object):

    def __init__(self, bucket, secret=None):
        self.bucket = boto.connect_s3().get_bucket(bucket)
        self.secret = secret

        self.url = 'http://' + self.bucket.get_website_endpoint()
        if secret:
            self.url += '/' + secret

    def _key(self, package, filename=None):
        path = '%s/%s' % (package.name, filename or package.filename)
        return Key(self.bucket, '%s/%s' % (self.secret, path) if self.secret else path)

    def get_index(self, package):
        try:
            html = self._key(package, 'index.html').get_contents_as_string()
            return Index.parse(self.url, html)
        except S3ResponseError:
            return Index(self.url, [])

    def put_index(self, package, index):
        k = self._key(package, 'index.html')
        k.set_metadata('Content-Type', 'text/html')
        k.set_metadata('Cache-Control', 'public, must-revalidate, proxy-revalidate, max-age=0')
        k.set_contents_from_string(index.to_html())
        k.set_acl('public-read')

    def put_package(self, package):
        k = self._key(package)
        k.set_metadata('Content-Type', 'application/x-gzip')
        k.set_contents_from_filename(os.path.join('dist', package.filename))
        k.set_acl('public-read')



#!/usr/bin/env python
import argparse

from s3pypi import __prog__, __version__
from s3pypi.package import Package
from s3pypi.storage import S3Storage


def main():
    p = argparse.ArgumentParser(prog=__prog__, version=__version__)
    p.add_argument('--bucket', required=True, help='S3 bucket')
    p.add_argument('--secret', help='S3 secret')
    args = p.parse_args()

    package = Package.create()
    storage = S3Storage(args.bucket, args.secret)

    index = storage.get_index(package)
    index.packages.add(package)

    storage.put_package(package)
    storage.put_index(package, index)


if __name__ == '__main__':
    main()
